/*************************************************************************************
 *  Copyright (C) 2008 by Aleix Pol <aleixpol@kde.org>                               *
 *                                                                                   *
 *  This program is free software; you can redistribute it and/or                    *
 *  modify it under the terms of the GNU General Public License                      *
 *  as published by the Free Software Foundation; either version 2                   *
 *  of the License, or (at your option) any later version.                           *
 *                                                                                   *
 *  This program is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of                   *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                    *
 *  GNU General Public License for more details.                                     *
 *                                                                                   *
 *  You should have received a copy of the GNU General Public License                *
 *  along with this program; if not, write to the Free Software                      *
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA   *
 *************************************************************************************/

#include "interactivegraphs.h"

#include <QDockWidget>
#include <QLayout>

#include "graph2d.h"
#include "variables.h"
#include "functionedit.h"
#include "functionsmodel.h"
#include "valuemodifier.h"

InteractiveGraphs::InteractiveGraphs(QWidget* parent) : QMainWindow(parent)
{
	m_fmodel=new FunctionsModel(this);
	m_graph=new Graph2D(m_fmodel, parent);
	setCentralWidget(m_graph);
	
	//Dock Edit
	QDockWidget *dock=new QDockWidget(this);
	m_edit=new FunctionEdit(dock);
	m_edit->setVariables(new Variables);
	qDebug () << "v" << m_edit->variables();
	dock->setWidget(m_edit);
	connect(m_edit, SIGNAL(accept()), this, SLOT(functionAccepted()));
	this->addDockWidget(Qt::RightDockWidgetArea, dock);
	
	//Dock Variables
	QDockWidget *dockValues=new QDockWidget(this);
	QWidget *values=new QWidget(dockValues);
	dockValues->setWidget(values);
	values->setLayout(new QVBoxLayout);
	
	ValueModifier *v = new ValueModifier("k", 0., values);
	variablesValue("k", 0.);
	connect(v, SIGNAL(modified(QString, double)), this, SLOT(variablesValue(QString, double)));
	values->layout()->addWidget(v);
	values->layout()->addItem(new QSpacerItem(0,0, QSizePolicy::Expanding, QSizePolicy::Expanding));
	this->addDockWidget(Qt::LeftDockWidgetArea, dockValues);
}

void InteractiveGraphs::functionAccepted()
{
	m_fmodel->clear();
	function f(m_edit->name(), Expression(m_edit->text(), m_edit->isMathML()), m_edit->variables(), m_edit->color());
	m_fmodel->addFunction(f);
	m_fmodel->setSelected(m_edit->name());
	qDebug() << "added" << m_edit->name() << "v:" << m_edit->variables();
}

void InteractiveGraphs::variablesValue(const QString& name, const double& value)
{
	qDebug() << "setting" << name << '=' << value;
	m_edit->variables()->modify(name, value);
	if(m_fmodel->rowCount()>0)
		m_graph->update(m_fmodel->index(0,0), m_fmodel->index(m_fmodel->rowCount()-1,0));
}
